/* $Id: teclado.h,v 1.2 2005/11/07 05:54:26 geni Exp $
 *
 * teclado.h
 *
 * Funciones auxiliares para manejar el teclado (y crear uno virtual)
 *
 * Historia:
 *      15/02/01 Pongo todo lo que es del teclado virtual aqu.
 *      22/02/01 Cambio el panel numrico y pongo un panel de "calculadora".
 *       9/06/01 Corrijo un bug en TecladoInit, que la condicin del while
 *               pona i-- en vez de --i, con lo que se haca vlida para i==-1
 *               e inicializaba mal Teclado->Actual.
 *      23/10/01 Hago que use el espacio de bffer de __Pantalla[] para
 *               cachear el dibujo del panel actual del teclado virtual para
 *               la profundidad inicial.
 *
 * Autor: Dario Rodriguez dario@softhome.net
 * This program is licensed under the terms of the GNU LGPL
 */


#ifndef _TECLADO_H_
#define _TECLADO_H_

/* Definicin de constantes */

#define TECLA_FILER 0x1
#define TECLA_MENU  0x2
#define TECLA_EL    0x4
#define TECLA_ENTER 0x8
#define TECLA_IZQ 0x10
#define TECLA_ARR 0x20
#define TECLA_DER 0x40
#define TECLA_ABJ 0x80

#define NUMPANELES 6
#define PANEL_ALFA 1
#define PANEL_ALFA2 2
#define PANEL_SYM  4
#define PANEL_SYM2 8
#define PANEL_NUM  16
#define PANEL_CALC 32
#define PANEL_TODOS (PANEL_ALFA|PANEL_ALFA2|PANEL_SYM|PANEL_SYM2|PANEL_NUM|PANEL_CALC)

#define TECLADO_BORRAR  8
#define TECLADO_ENTER  13

#ifndef NULL
#define NULL ((void *) 0L)
#endif

/* Definicin de estructuras y datos globales */

#ifdef _TECLADO_C_
char __Pantalla[8*102+4];
char *__PantallaPanelPtr=NULL;
char __ListaPaneles[]={PANEL_ALFA,PANEL_ALFA2,PANEL_SYM,PANEL_SYM2,PANEL_NUM,PANEL_CALC};
#else
extern char __Pantalla[], *__PantallaPanelPtr, __ListaPaneles[];
#endif


typedef struct TipoTeclado {
        int MascaraPaneles;
        int NumPanel;
        int Actual[NUMPANELES];
} sTeclado;

struct tmp {
        int Mascara;
        char *Opciones;
        int Prof;
};

#ifdef _TECLADO_C_
struct tmp __Paneles[]={
        {PANEL_ALFA,"ABCDEFGHIJKLMNOPQRSTUVWXYZ   ^_<   ",2},
        {PANEL_ALFA2,"abcdefghijklmnopqrstuvwxyz   ^_<   ",2},
        {PANEL_SYM,":;_=/#$ܪ\"%[\\]{@}___   ^\254<   ",2},
        {PANEL_SYM2,".,-+*&|!?`'<^>(~)___   ^\254<   ",2},
        {PANEL_NUM, "1234567890.<",1},
        {PANEL_CALC,"[*](+){/}=-<",1},
        {0,(char *)0,0}};
#else
extern struct tmp __Paneles[];
#endif

/* Prototipos de funciones: funciones pblicas */

int EsperaTecla(int Repeat);
void TecladoInit(sTeclado *Teclado, int MascaraPaneles, int PanelInicial);
int TecladoLee(sTeclado *Teclado);

/* Prototipos de funciones: funciones privadas (auxiliares de las anteriores) */

int ProcesaPantalla(int Guardar);
int LeeLetra(char *Opciones, int ProfIni);
void PintaOpciones(char *Opciones, int Tipo, int Desp, int NumOpcionesOmitir);

/*
 * tinyfont.h
 *
 * Funciones de utilidad para el uso de tinyfont.
 *
 * Historia:
 *      15/02/01 Pongo todo esto es un fichero aparte.
 *      22/04/01 Cambio de nombre a PonIntPeq() a PonIntPeqInfo(), le hago que
 *               devuelva el nmero de carcteres escritos y hago una macro
 *               con el nombre de PonIntPeq para no perder la compatibilidad.
 *       5/12/01 Cambios cosmticos para que compile sin avisos con -Wall.
 *
 * Autor: Dario Rodriguez dario@softhome.net
 * This program is licensed under the terms of the GNU LGPL
 */

/* Definicin de constantes */

#ifndef COPY_PUT
#define COPY_PUT 0
#define XOR_PUT  1
#define OR_PUT   2
#define AND_PUT  3
#endif

#ifndef KCOPY_PUT
#define KCOPY_PUT 0
#define KOR_PUT   1
#endif

#ifndef PCOPY_PUT
#define PCOPY_PUT (COPY_PUT | 4)
#define PXOR_PUT  (XOR_PUT | 4)
#define POR_PUT   (OR_PUT | 4)
#define PAND_PUT  (AND_PUT | 4)
#endif


/* Definicin de estructuras y datos globales */

#ifdef _TECLADO_C_
char Fuente[128*8+4];
#else
extern char Fuente[];
#endif

/* Definicin de macros */

#ifndef LINUX
#define TinyfontInit() mpread(Fuente,"\\system\\mmp\\tinyfont.mmp")
#define PonLetraPeq(x,y,Letra) gv_aput(x,y,4,6,\
        (unsigned long) (Fuente+((unsigned short) Letra)*4),COPY_PUT)
#else
#define TinyfontInit() mpread(Fuente,"tinyfont.mmp")
#define gv_aput_inv(op, dir, ty, tx, y, x) gv_aput(x,y,tx,ty,dir,op)
#define PonLetraPeq(x,y,Letra) gv_aput_inv(COPY_PUT, \
        (unsigned long) (Fuente+((unsigned short) Letra)*4), 6,4,y,x)
#endif
#define PonIntPeq(a,b,c) PonIntPeqInfo(a,b,c,NULL)

/* Prototipos de funciones */

void PonCadPeq(int x,int y, char *Cad);
void PonIntPeqInfo(int x,int y, unsigned long Num,int *NumCar);
int IntPeqInfo(unsigned long Num);

/* posiciones.h */


/*
0 (0.500000)
4 (4.896179)
6 (6.580479)
7 (7.775816)
8 (8.698916)
9 (9.434561)
10 (10.026279)
10 (10.499653)
10 (10.870899)
11 (11.150704)
11 (11.346146)
11 (11.461739)
11 (11.500000)
11 (11.461739)
11 (11.346146)
11 (11.150704)
10 (10.870899)
10 (10.499653)
10 (10.026279)
9 (9.434561)
8 (8.698916)
7 (7.775816)
6 (6.580479)
4 (4.896179)
0 (0.500000)
*/
#ifdef _TECLADO_C_
char posiciones9[] = { /* x1,y1,x2,y2,x3,y3,...*/ 
/* izq */ 18, 53, 14, 52, 10, 51, 6, 49, 2, 45, 6, 41, 10, 39, 14, 38, 18, 37, 
/* arr */ 34, 35, 38, 34, 42, 34, 46, 34, 50, 34, 54, 34, 58, 34, 62, 34, 66, 35, 
/* der */ 81, 37, 85, 38, 89, 39, 93, 41, 97, 45, 93, 49, 89, 51, 85, 52, 81, 53, 
/* abj */ 66, 55, 62, 56, 58, 56, 54, 56, 50, 56, 46, 56, 42, 56, 38, 56, 34, 55, };

char posiciones3[] = { /* x1,y1,x2,y2,x3,y3,...*/ 
/* izq */ 30, 49, 26, 45, 30, 41, 
/* arr */ 46, 34, 50, 34, 54, 34, 
/* der */ 70, 41, 74, 45, 70, 49, 
/* abj */ 54, 56, 50, 56, 46, 56, };

char posiciones1[] = { /* x1,y1,x2,y2,x3,y3,...*/ 
/* izq */ 42, 45, 
/* arr */ 50, 34, 
/* der */ 58, 45, 
/* abj */ 50, 56, };
#else
extern char posiciones9[], posiciones3[], posiciones1[];
#endif

enum EnumBanderasFichSel {
        fichDirectorio=1,
        fichSeleccionado=2
};

typedef struct TipoFileSelElem {
        char Tipo;
        char Nombre[8+1+3+1];
        unsigned char Dia;
        unsigned char Mes;
        unsigned char Anio;
} sFileSelElem;

typedef struct TipoFileSelDir {
        int NumElemSel;
        int Ocupados;
        sFileSelElem Elem[5];
} sFileSelDir;

enum EnumAccionFileSelDir {
        dirInit=0,
        dirSiguiente,
        dirAnterior,
        dirCompleta,
        dirRepinta,
};

enum EnumGuiActivo {
        guiNombre=0,
        guiBotones,
        guiLista
};

typedef enum EnumDialogBox {
        boxOk=0,
        boxOkCancel,
        boxYesNo,
} eDialogBox;

typedef struct TipoMenu {
        char *String;
        int Space;
        int (*fn)(int PosX, void *Data);
        void *Data;
} sMenu;


int MenuInit(sMenu *Menu);
int MenuVert(int x, int y, sMenu *Menu);
int fnSubMenu(int PosX, void *Data);
void sombrea(int x1,int y1, int x2);
void procesafondo(int Guardar);

#ifndef ABS
#define ABS(x) (((x)<0)?(-(x)):(x))
#endif


char *FileSel(char *Titulo, char *BufferSeleccion, int TamBuffer);
void DibujaVentana(char *Texto);
void DibujaFileSel(char *Texto);
void DibujaBotones(int NumSel);
void DibujaBoton(int x1, int y1, int x2, int y2, char *Texto, char BanderaSel);
void DibujaNombre(char *Nombre, char BanderaCursor);
void DibujaFichSel(int Pos,sFileSelElem *Elem);
void DibujaFichSelCursor(int Pos, char BanderaEstado);
void DibujaNombreDir(char *Texto);
void FileSelDirRellena(int Accion,sFileSelDir *Dir, char *Path);
void FileSelDirPinta(sFileSelDir *Dir, char *Path);
int FileSelDirProcesa(sFileSelDir *Dir, char *Path);
void NombreProcesa(char *Nombre, sTeclado *Teclado);

int DialogBox(char *Titulo,char *Mensaje, eDialogBox Tipo);
void DibujaBotonesDialogBox(int NumSel,char **Textos);

/* Funciones auxiliares */
char *fs_copiacad(char *Dest, char*Orig, int TamDest);
char *fs_copiamem(char *Dest, char*Orig, int NumBytes);
char *fs_initmem(char *Dest, int c, int NumBytes);
char *fs_strrchr(char *Cad, int c);
int fs_cmpcadmay(char *a, char *b);
void DTA2SelElem(char *DTA, sFileSelElem *Elem);
int cmpSelElem(sFileSelElem *a,sFileSelElem *b);


#endif
